@@ -113,8 +113,8 @@ struct assh_channel_s
   struct assh_queue_s request_lqueue; //< requests waiting for a reply from the remote host
 
   uint32_t remote_id;
-  uint32_t rpkt_size;		//< remote packet size
-  uint32_t lpkt_size;		//< local packet size
+  uint32_t rpkt_size;		//< remote max accepted packet size
+  uint32_t lpkt_size;		//< local max accepted packet size
 
   uint32_t rwin_left;           //< remote window bytes left
   uint32_t lwin_left;           //< local window bytes left
@@ -1121,7 +1121,7 @@ assh_connection_got_channel_open(struct assh_session_s *s,
 
   ch->mentry.id = assh_channel_next_id(pv);
   ch->remote_id = rid;
-  ch->rpkt_size = pkt_size;
+  ch->rpkt_size = assh_min_uint(pkt_size, ASSH_CHANNEL_MAX_PKTSIZE);
   ch->rwin_left = win_size;
   ASSH_SET_STATE(ch, state, ASSH_CHANNEL_ST_OPEN_RECEIVED);
   ch->session = s;
@@ -1150,7 +1150,7 @@ assh_connection_got_channel_open(struct assh_session_s *s,
   ev->win_size = -1;
   ev->pkt_size = -1;
   ev->rwin_size = win_size;
-  ev->rpkt_size = pkt_size;
+  ev->rpkt_size = ch->rpkt_size;
 
   struct assh_cbuffer_s *rq_data = &ev->rq_data;
   rq_data->size = p->data + p->data_size - data;
@@ -1318,9 +1318,12 @@ assh_connection_got_channel_open_reply(struct assh_session_s *s,
     {
       ASSH_RET_ON_ERR(assh_packet_check_u32(p, &ch->remote_id, data, &data));
       ASSH_RET_ON_ERR(assh_packet_check_u32(p, &ch->rwin_left, data, &data));
-      ASSH_RET_ON_ERR(assh_packet_check_u32(p, &ch->rpkt_size, data, &data));
 
-      ASSH_RET_IF_TRUE(ch->rpkt_size < 1, ASSH_ERR_PROTOCOL);
+      uint32_t rpkt_size;
+      ASSH_RET_ON_ERR(assh_packet_check_u32(p, &rpkt_size, data, &data));
+      ASSH_RET_IF_TRUE(rpkt_size < 1, ASSH_ERR_PROTOCOL);
+
+      ch->rpkt_size = assh_min_uint(rpkt_size, ASSH_CHANNEL_MAX_PKTSIZE);
 
       struct assh_event_channel_confirmation_s *ev =
         &e->connection.channel_confirmation;
@@ -1566,6 +1569,7 @@ assh_connection_got_channel_data(struct assh_session_s *s,
   ev->data.size = size;
   ev->data.data = (uint8_t*)data;
   ev->transferred = 0;
+  ev->win_size = ch->lwin_left;
 
   /* keep packet for data buffer */
   pv->in_data_left = size;
@@ -1578,6 +1582,13 @@ assh_connection_got_channel_data(struct assh_session_s *s,
   return ASSH_OK;
 }
 
+static ASSH_EVENT_DONE_FCN(assh_event_channel_window_adjust_done)
+{
+  assert(s->srv == &assh_service_connection);
+
+  return ASSH_OK;
+}
+
 static ASSH_WARN_UNUSED_RESULT assh_status_t
 assh_connection_got_channel_window_adjust(struct assh_session_s *s,
                                           struct assh_packet_s *p,
@@ -1628,7 +1639,7 @@ assh_connection_got_channel_window_adjust(struct assh_session_s *s,
 
   /* setup event */
   e->id = ASSH_EVENT_CHANNEL_WINDOW;
-  e->f_done = NULL;
+  e->f_done = assh_event_channel_window_adjust_done;
 
   ev->ch = ch;
   ev->old_size = ch->rwin_left;
@@ -1883,6 +1894,35 @@ assh_channel_dummy(struct assh_channel_s *ch, size_t size)
   return err;
 }
 
+assh_status_t
+assh_channel_overhead(struct assh_session_s *s, size_t *data_size,
+		      size_t *packet_size, assh_bool_t ext)
+{
+  assh_status_t err;
+  size_t chan_data_header = ext ? 3 * 4 : 2 * 4;
+  size_t payload_size;
+
+  if (*packet_size)
+    {
+      ASSH_RET_ON_ERR(assh_transport_overhead(s, &payload_size, packet_size));
+
+      ASSH_RET_IF_TRUE(payload_size <= chan_data_header,
+		       ASSH_ERR_OUTPUT_OVERFLOW);
+
+      *data_size = payload_size - chan_data_header;
+
+      return ASSH_OK;
+    }
+  else if (*data_size)
+    {
+      payload_size = *data_size + chan_data_header;
+
+      ASSH_RETURN(assh_transport_overhead(s, &payload_size, packet_size));
+    }
+
+  ASSH_RETURN(ASSH_ERR_BAD_ARG);
+}
+
 /************************************************* incoming channel close/eof */
 
 static ASSH_WARN_UNUSED_RESULT assh_status_t
