@@ -115,8 +115,6 @@ assh_kex_list(struct assh_session_s *s, struct assh_packet_s *p,
   ASSH_DEBUG_("\n");
 #endif
 
-  ASSH_RET_IF_TRUE(!first, ASSH_ERR_MISSING_ALGO);
-
   return ASSH_OK;
 }
 
@@ -125,14 +123,16 @@ assh_status_t assh_kex_send_init(struct assh_session_s *s)
   assh_status_t err;
   struct assh_context_s *c = s->ctx;
 
+  /* the appropriate packet size is computed
+     in assh_algo_kex_init_size */
   struct assh_packet_s *p;
   ASSH_RET_ON_ERR(assh_packet_alloc(c, SSH_MSG_KEXINIT,
                  c->kex_init_size, &p));
 
   uint8_t *cookie;
-  ASSH_JMP_ON_ERR(assh_packet_add_array(p, 16, &cookie), err_pck);
-  ASSH_JMP_ON_ERR(assh_prng_get(c, cookie,
-		  16, ASSH_PRNG_QUALITY_NONCE), err_pck);
+  ASSH_ASSERT(assh_packet_add_array(p, 16, &cookie));
+  ASSH_ASSERT(assh_prng_get(c, cookie,
+		  16, ASSH_PRNG_QUALITY_NONCE));
 
   /* lists of algorithms */
   uint_fast16_t i = 0, j;
@@ -148,10 +148,10 @@ assh_status_t assh_kex_send_init(struct assh_session_s *s)
   uint8_t *x;
 
   /* empty languages */
-  ASSH_JMP_ON_ERR(assh_packet_add_string(p, 0, &x), err_pck);
-  ASSH_JMP_ON_ERR(assh_packet_add_string(p, 0, &x), err_pck);
+  ASSH_ASSERT(assh_packet_add_string(p, 0, &x));
+  ASSH_ASSERT(assh_packet_add_string(p, 0, &x));
 
-  ASSH_JMP_ON_ERR(assh_packet_add_array(p, 5, &x), err_pck);
+  ASSH_ASSERT(assh_packet_add_array(p, 5, &x));
 
   /* fkpf + reserved */
   memset(x, 0, 5);
@@ -435,8 +435,11 @@ assh_status_t assh_kex_got_init(struct assh_session_s *s, struct assh_packet_s *
 
   /* alloacte input and output keys and associated cipher/mac/compress contexts */
   struct assh_kex_keys_s *kin;
-  size_t kin_size = sizeof(*kin) + ca_in->ctx_size + cpa_in->ctx_size
-    + ma_in->ctx_size;
+  size_t kin_size = ASSH_STRUCT_ALIGN(sizeof(*kin))
+    + ASSH_STRUCT_ALIGN(ca_in->ctx_size)
+    + ASSH_STRUCT_ALIGN(cpa_in->ctx_size)
+    + ASSH_STRUCT_ALIGN(ma_in->ctx_size)
+    + ASSH_STRUCT_ALIGN(1);
   ASSH_RET_ON_ERR(assh_alloc(s->ctx, kin_size, ASSH_ALLOC_SECUR, (void**)&kin));
 
   kin_safety = assh_min_uint(kin_safety, ca_in->algo.safety);
@@ -444,8 +447,11 @@ assh_status_t assh_kex_got_init(struct assh_session_s *s, struct assh_packet_s *
     kin_safety = assh_min_uint(kin_safety, ma_in->algo.safety);
 
   struct assh_kex_keys_s *kout;
-  size_t kout_size = sizeof(*kout) + ca_out->ctx_size + cpa_out->ctx_size
-    + ma_out->ctx_size;
+  size_t kout_size = ASSH_STRUCT_ALIGN(sizeof(*kout))
+    + ASSH_STRUCT_ALIGN(ca_out->ctx_size)
+    + ASSH_STRUCT_ALIGN(cpa_out->ctx_size)
+    + ASSH_STRUCT_ALIGN(ma_out->ctx_size)
+    + ASSH_STRUCT_ALIGN(1);
   ASSH_JMP_ON_ERR(assh_alloc(s->ctx, kout_size, ASSH_ALLOC_SECUR, (void**)&kout),
 	       err_kin);
 
@@ -591,8 +597,8 @@ assh_kex_new_keys(struct assh_session_s *s,
 
   struct assh_kex_keys_s *kin = s->new_keys_in;
   struct assh_kex_keys_s *kout = s->new_keys_out;
-  uint8_t *next_in_ctx = (void*)(kin + 1);
-  uint8_t *next_out_ctx = (void*)(kout + 1);
+  uint8_t *next_in_ctx = (uint8_t*)ASSH_STRUCT_ALIGN((uintptr_t)(kin + 1));
+  uint8_t *next_out_ctx = (uint8_t*)ASSH_STRUCT_ALIGN((uintptr_t)(kout + 1));
 
   /* get input cipher iv/key and init cipher */
   if (kin->cipher_algo->iv_size)
@@ -608,7 +614,7 @@ assh_kex_new_keys(struct assh_session_s *s,
   c++;
 
   kin->cipher_ctx = (void*)next_in_ctx;
-  next_in_ctx += kin->cipher_algo->ctx_size;
+  next_in_ctx += ASSH_STRUCT_ALIGN(kin->cipher_algo->ctx_size);
 
 #ifdef CONFIG_ASSH_DEBUG_KEX
   ASSH_DEBUG_HEXDUMP("in iv", iv, kin->cipher_algo->iv_size);
@@ -631,7 +637,7 @@ assh_kex_new_keys(struct assh_session_s *s,
   c++;
 
   kout->cipher_ctx = (void*)next_out_ctx;
-  next_out_ctx += kout->cipher_algo->ctx_size;
+  next_out_ctx += ASSH_STRUCT_ALIGN(kout->cipher_algo->ctx_size);
 
 #ifdef CONFIG_ASSH_DEBUG_KEX
   ASSH_DEBUG_HEXDUMP("out iv", iv, kout->cipher_algo->iv_size);
@@ -651,7 +657,7 @@ assh_kex_new_keys(struct assh_session_s *s,
 #endif
     }
   kin->mac_ctx = (void*)next_in_ctx;
-  next_in_ctx += kin->mac_algo->ctx_size;
+  next_in_ctx += ASSH_STRUCT_ALIGN(kin->mac_algo->ctx_size);
   ASSH_JMP_ON_ERR(kin->mac_algo->f_init(s->ctx, kin->mac_ctx, key, 0), err_mac_in);
   c++;
 
@@ -666,7 +672,7 @@ assh_kex_new_keys(struct assh_session_s *s,
 #endif
     }
   kout->mac_ctx = (void*)next_out_ctx;
-  next_out_ctx += kout->mac_algo->ctx_size;
+  next_out_ctx += ASSH_STRUCT_ALIGN(kout->mac_algo->ctx_size);
   ASSH_JMP_ON_ERR(kout->mac_algo->f_init(s->ctx, kout->mac_ctx, key, 1), err_mac_out);
   c++;
 
