@@ -74,6 +74,7 @@ void assh_transport_push(struct assh_session_s *s,
 
     case ASSH_TR_SERVICE:
     case ASSH_TR_DISCONNECT:
+      s->queue_out_size += p->data_size;
       assh_queue_push_back(q, &p->entry);
       break;
 
@@ -83,7 +84,9 @@ void assh_transport_push(struct assh_session_s *s,
     }
 }
 
-static ASSH_EVENT_DONE_FCN(assh_event_read_done)
+assh_status_t
+assh_transport_input_done(struct assh_session_s *s,
+			  size_t rd_size)
 {
   assh_status_t err;
   struct assh_kex_keys_s *k = s->cur_keys_in;
@@ -91,15 +94,6 @@ static ASSH_EVENT_DONE_FCN(assh_event_read_done)
   const struct assh_algo_cipher_s *ca = k->cipher_algo;
   uint_fast8_t hsize = ca->head_size;
 
-  if (inerr)
-    {
-      ASSH_SET_STATE(s, stream_in_st, ASSH_TR_IN_CLOSED);
-      ASSH_RETURN(inerr | ASSH_ERRSV_DISCONNECT);
-    }
-
-  size_t rd_size = e->transport.read.transferred;
-
-  assert(rd_size <= e->transport.read.buf.size);
   s->stream_in_size += rd_size;
 
   switch (s->stream_in_st)
@@ -146,7 +140,7 @@ static ASSH_EVENT_DONE_FCN(assh_event_read_done)
 		   ASSH_ERR_INPUT_OVERFLOW | ASSH_ERRSV_DISCONNECT);
 
       ASSH_SET_STATE(s, stream_in_st, ASSH_TR_IN_IDENT);
-      return ASSH_OK;
+      return ASSH_NO_DATA;
     }
 
     /* decipher packet head, compute packet length and allocate packet */
@@ -157,7 +151,7 @@ static ASSH_EVENT_DONE_FCN(assh_event_read_done)
 	{
 	  /* not enough header data yet to decipher the 1st block */
 	  ASSH_SET_STATE(s, stream_in_st, ASSH_TR_IN_HEAD);
-	  return ASSH_OK;
+	  return ASSH_NO_DATA;
 	}
 
       /* decipher head */
@@ -169,27 +163,25 @@ static ASSH_EVENT_DONE_FCN(assh_event_read_done)
       /* check length */
       size_t len = assh_load_u32(s->stream_in_stub.head.pck_len);
 
-      ASSH_RET_IF_TRUE(len - ASSH_PACKET_MIN_PADDING - 1 > CONFIG_ASSH_MAX_PAYLOAD,
+      ASSH_RET_IF_TRUE(len < 1 + ASSH_PACKET_MIN_PADDING,
+		   ASSH_ERR_INPUT_OVERFLOW | ASSH_ERRSV_DISCONNECT);
+
+      size_t mac_len = ma->mac_size + ca->auth_size;
+      size_t data_size = /* pck_len field */ 4 + len + mac_len;
+
+      ASSH_RET_IF_TRUE(data_size > CONFIG_ASSH_MAX_PACKET_LEN,
 		   ASSH_ERR_INPUT_OVERFLOW | ASSH_ERRSV_DISCONNECT);
 
       /* allocate actual packet and copy header */
-      size_t mac_len = ma->mac_size + ca->auth_size;
-      size_t buffer_size = /* pck_len field */ 4 + len;
-
-      ASSH_RET_IF_TRUE(buffer_size < ca->block_size,
-      		       ASSH_ERR_INPUT_OVERFLOW | ASSH_ERRSV_DISCONNECT);
-
-      buffer_size += mac_len;
-
-      ASSH_RET_IF_TRUE(buffer_size < s->stream_in_size,
+      ASSH_RET_IF_TRUE(data_size < s->stream_in_size,
 		   ASSH_ERR_INPUT_OVERFLOW | ASSH_ERRSV_DISCONNECT);
 
       struct assh_packet_s *p;
-      ASSH_RET_ON_ERR(assh_packet_alloc_raw(s->ctx, buffer_size, &p)
+      ASSH_RET_ON_ERR(assh_packet_alloc_raw(s->ctx, data_size, &p)
 		   | ASSH_ERRSV_DISCONNECT);
 
       memcpy(p->data, s->stream_in_stub.data, s->stream_in_size);
-      p->data_size = buffer_size;
+      p->data_size = data_size;
       s->stream_in_pck = p;
     }
 
@@ -201,7 +193,7 @@ static ASSH_EVENT_DONE_FCN(assh_event_read_done)
 	{
 	  /* not enough data for the whole packet yet */
 	  ASSH_SET_STATE(s, stream_in_st, ASSH_TR_IN_PAYLOAD);
-	  return ASSH_OK;
+	  return ASSH_NO_DATA;
 	}
 
       uint32_t seq = s->in_seq;
@@ -212,12 +204,24 @@ static ASSH_EVENT_DONE_FCN(assh_event_read_done)
 
       if (ca->auth_size)	/* Authenticated cipher */
 	{
+	  ASSH_RET_IF_TRUE(data_size < 4 + ca->block_size + mac_len,
+			   ASSH_ERR_INPUT_OVERFLOW | ASSH_ERRSV_DISCONNECT);
+
+	  ASSH_RET_IF_TRUE((data_size - 4 - mac_len) & (ca->block_size - 1),
+			   ASSH_ERR_INPUT_OVERFLOW | ASSH_ERRSV_DISCONNECT);
+
 	  ASSH_RET_ON_ERR(ca->f_process(k->cipher_ctx, data,
 				       data_size, ASSH_CIPHER_PCK_TAIL, seq)
 		       | ASSH_ERRSV_DISCONNECT);
 	}
       else if (ma->etm)	/* Encrypt then Mac */
 	{
+	  ASSH_RET_IF_TRUE(data_size < 4 + ca->block_size + mac_len,
+			   ASSH_ERR_INPUT_OVERFLOW | ASSH_ERRSV_DISCONNECT);
+
+	  ASSH_RET_IF_TRUE((data_size - 4 - mac_len) & (ca->block_size - 1),
+			   ASSH_ERR_INPUT_OVERFLOW | ASSH_ERRSV_DISCONNECT);
+
 	  ASSH_RET_ON_ERR(ma->f_process(k->mac_ctx, data,
 					    data_size - mac_len,
 					    data + data_size - mac_len, seq)
@@ -227,8 +231,11 @@ static ASSH_EVENT_DONE_FCN(assh_event_read_done)
 				  data_size - mac_len - 4, ASSH_CIPHER_PCK_TAIL, seq)
 		       | ASSH_ERRSV_DISCONNECT);
 	}
-      else			/* Mac and Encrypt */
+      else if (data_size > hsize + mac_len)	/* Mac and Encrypt */
 	{
+	  ASSH_RET_IF_TRUE((data_size - mac_len) & (ca->block_size - 1),
+			   ASSH_ERR_INPUT_OVERFLOW | ASSH_ERRSV_DISCONNECT);
+
 	  ASSH_RET_ON_ERR(ca->f_process(k->cipher_ctx, data + hsize,
 				  data_size - hsize - mac_len, ASSH_CIPHER_PCK_TAIL, seq)
 		       | ASSH_ERRSV_DISCONNECT);
@@ -240,32 +247,42 @@ static ASSH_EVENT_DONE_FCN(assh_event_read_done)
 	}
 
       /* check and adjust packet data size */
-      size_t len = assh_load_u32(p->head.pck_len);
       uint8_t pad_len = p->head.pad_len;
 
       ASSH_RET_IF_TRUE(pad_len < 4,
 		   ASSH_ERR_INPUT_OVERFLOW | ASSH_ERRSV_DISCONNECT);
 
-      ASSH_RET_IF_TRUE(len < /* pad_len field */ 1 + /* msg field */ 1 + pad_len,
-		   ASSH_ERR_INPUT_OVERFLOW | ASSH_ERRSV_DISCONNECT);
+      data_size -= mac_len;
 
-      ASSH_RET_IF_TRUE(len - pad_len - 1 > CONFIG_ASSH_MAX_PAYLOAD,
-		   ASSH_ERR_INPUT_OVERFLOW | ASSH_ERRSV_DISCONNECT);
+      ASSH_RET_IF_TRUE(data_size < ASSH_PACKET_HEADLEN + 1 + pad_len,
+		       ASSH_ERR_INPUT_OVERFLOW | ASSH_ERRSV_DISCONNECT);
 
-      p->data_size = data_size - mac_len - pad_len;
+      data_size -= pad_len;
+      p->data_size = data_size;
+
+      if (k->cmp_algo != &assh_compress_none)
+	{
+	  /* decompress payload */
+	  struct assh_packet_s *p_ = p;
+	  ASSH_RET_ON_ERR(k->cmp_algo->f_process(s->ctx, k->cmp_ctx, &p,
+			    s->tr_user_auth_done, 0) | ASSH_ERRSV_DISCONNECT);
+
+	  if (p_ != p)
+	    assh_packet_release(p_);
+
+	  data_size = p->data_size;
+
+	  ASSH_RET_IF_TRUE(data_size < ASSH_PACKET_HEADLEN + 1,
+			   ASSH_ERR_INPUT_OVERFLOW | ASSH_ERRSV_DISCONNECT);
+	}
+
+      ASSH_RET_IF_TRUE(data_size > ASSH_PACKET_HEADLEN + CONFIG_ASSH_MAX_PAYLOAD_LEN,
+		       ASSH_ERR_INPUT_OVERFLOW | ASSH_ERRSV_DISCONNECT);
 
       /* push completed incoming packet for dispatch */
       p->seq = seq;
       assert(s->in_pck == NULL);
 
-      /* decompress payload */
-      struct assh_packet_s *p_ = p;
-      ASSH_RET_ON_ERR(k->cmp_algo->f_process(s->ctx, k->cmp_ctx, &p, s->tr_user_auth_done)
-		   | ASSH_ERRSV_DISCONNECT);
-
-      if (p_ != p)
-	assh_packet_release(p_);
-
 #ifdef CONFIG_ASSH_DEBUG_PROTOCOL
       ASSH_DEBUG("incoming packet: session=%p tr_st=%i, size=%zu, msg=%u\n",
 		 s, s->tr_st, data_size, p->head.msg);
@@ -293,12 +310,11 @@ static ASSH_EVENT_DONE_FCN(assh_event_read_done)
     }
 }
 
-assh_status_t assh_transport_read(struct assh_session_s *s,
-				 struct assh_event_s *e)
+assh_status_t
+assh_transport_input_buffer(struct assh_session_s *s,
+			    uint8_t **data, size_t *size)
 {
   struct assh_kex_keys_s *k = s->cur_keys_in;
-  uint8_t **data = &e->transport.read.buf.data;
-  size_t *size = &e->transport.read.buf.size;
 
   if (s->tr_st >= ASSH_TR_DISCONNECT)
     ASSH_SET_STATE(s, stream_in_st, ASSH_TR_IN_CLOSED);
@@ -313,64 +329,49 @@ assh_status_t assh_transport_read(struct assh_session_s *s,
 	 must not span more than one binary packet. */
       *size = assh_min_uint(ASSH_MIN_BLOCK_SIZE,
 			    sizeof(s->ident_str) - s->stream_in_size);
-      break;
+      return ASSH_OK;
 
     /* read stream into packet head buffer */
     case ASSH_TR_IN_HEAD: {
       *data = s->stream_in_stub.data + s->stream_in_size;
       ASSH_SET_STATE(s, stream_in_st, ASSH_TR_IN_HEAD_DONE);
       *size = k->cipher_algo->head_size - s->stream_in_size;
-      break;
+      return ASSH_OK;
     }
 
     /* read stream into actual packet buffer */
     case ASSH_TR_IN_PAYLOAD: {
+      if (s->in_pck != NULL)
+	return ASSH_NO_DATA;
+
       struct assh_packet_s *p = s->stream_in_pck;
       *data = p->data + s->stream_in_size;
       *size = p->data_size - s->stream_in_size;
       ASSH_SET_STATE(s, stream_in_st, ASSH_TR_IN_PAYLOAD_DONE);
-      assert(s->in_pck == NULL);
-      break;
-    }
-
-    case ASSH_TR_IN_CLOSED:
       return ASSH_OK;
+    }
 
     default:
-      ASSH_UNREACHABLE();
+      return ASSH_NO_DATA;
     }
-
-  e->id = ASSH_EVENT_READ;
-  e->f_done = &assh_event_read_done;
-  e->transport.read.transferred = 0;
-  return ASSH_OK;
 }
 
-static ASSH_EVENT_DONE_FCN(assh_event_write_done)
+void
+assh_transport_output_done(struct assh_session_s *s,
+			   size_t wr_size, assh_bool_t yield)
 {
-  assh_status_t err;
-
-  if (inerr)
-    {
-      ASSH_SET_STATE(s, stream_out_st, ASSH_TR_OUT_CLOSED);
-      ASSH_RETURN(inerr);
-    }
-
-  size_t wr_size = e->transport.write.transferred;
-
-  assert(wr_size <= e->transport.write.buf.size);
-  s->stream_out_size += wr_size;
+  size_t ss = s->stream_out_size += wr_size;
 
   switch (s->stream_out_st)
     {
     /* check if sending of ident string has completed */
     case ASSH_TR_OUT_IDENT_DONE:
-      if (s->stream_out_size >= sizeof(ASSH_IDENT) - 1)
+      if (ss >= sizeof(ASSH_IDENT) - 1)
 	ASSH_SET_STATE(s, stream_out_st, ASSH_TR_OUT_PACKETS);
-      else
+      else if (yield)
 	ASSH_SET_STATE(s, stream_out_st, ASSH_TR_OUT_IDENT_PAUSE);
 
-      return ASSH_OK;
+      return;
 
     /* check if sending of packet has completed */
     case ASSH_TR_OUT_PACKETS_DONE: {
@@ -379,12 +380,13 @@ static ASSH_EVENT_DONE_FCN(assh_event_write_done)
       struct assh_queue_entry_s *e = assh_queue_front(&s->out_queue);
       struct assh_packet_s *p = (void*)e;
 
-      if (s->stream_out_size < p->data_size)
+      if (ss < p->data_size)
 	{
 	  /* packet partially sent, need to report one more write
 	     event later. Yield to the input state machine for now. */
-	  ASSH_SET_STATE(s, stream_out_st, ASSH_TR_OUT_PACKETS_PAUSE);
-	  return ASSH_OK;
+	  if (yield)
+	    ASSH_SET_STATE(s, stream_out_st, ASSH_TR_OUT_PACKETS_PAUSE);
+	  return;
 	}
 
       p->seq = s->out_seq++;
@@ -397,9 +399,10 @@ static ASSH_EVENT_DONE_FCN(assh_event_write_done)
 
       /* pop and release packet */
       assh_queue_remove(e);
+      s->queue_out_size -= p->data_size;
       assh_packet_release(p);
 
-      return ASSH_OK;
+      return;
     }
 
     default:
@@ -407,12 +410,12 @@ static ASSH_EVENT_DONE_FCN(assh_event_write_done)
     }
 }
 
-assh_status_t assh_transport_write(struct assh_session_s *s,
-				  struct assh_event_s *e)
+assh_status_t
+assh_transport_output_buffer(struct assh_session_s *s,
+			     uint8_t const ** const data,
+			     size_t *size)
 {
   assh_status_t err;
-  uint8_t const ** const data = &e->transport.write.buf.data;
-  size_t *size = &e->transport.write.buf.size;
 
   switch (s->stream_out_st)
     {
@@ -422,18 +425,19 @@ assh_status_t assh_transport_write(struct assh_session_s *s,
 	  s->tr_st < ASSH_TR_DISCONNECT)
 	ASSH_SET_STATE(s, stream_out_st, ASSH_TR_OUT_IDENT);
 
-      return ASSH_OK;
+      return ASSH_NO_DATA;
 
     /* the write stream buffer is the constant ident string */
     case ASSH_TR_OUT_IDENT: {
       if (s->stream_in_st == ASSH_TR_IN_CLOSED ||
 	  s->tr_st >= ASSH_TR_DISCONNECT)
-	return ASSH_OK;
+	return ASSH_NO_DATA;
 
       *data = (uint8_t*)ASSH_IDENT + s->stream_out_size;
       *size = sizeof(ASSH_IDENT) - 1 - s->stream_out_size;
       ASSH_SET_STATE(s, stream_out_st, ASSH_TR_OUT_IDENT_DONE);
-      break;
+
+      return ASSH_OK;
     }
 
     /* the next output packet must be enciphered before write */
@@ -447,7 +451,7 @@ assh_status_t assh_transport_write(struct assh_session_s *s,
 	{
 	  /* nothing to output, yield to input */
 	  if (assh_queue_isempty(q))
-	    return ASSH_OK;
+	    return ASSH_NO_DATA;
 
 	  p = (void*)assh_queue_front(q);
 	  msg = p->head.msg;
@@ -458,6 +462,7 @@ assh_status_t assh_transport_write(struct assh_session_s *s,
 	    break;
 
 	  assh_queue_remove(&p->entry);
+	  s->queue_out_size -= p->data_size;
 	  assh_packet_release(p);
 	}
 
@@ -471,21 +476,50 @@ assh_status_t assh_transport_write(struct assh_session_s *s,
       ASSH_DEBUG_HEXDUMP("out packet", p->data, p->data_size);
 #endif
 
-      struct assh_packet_s *p_ = p;
-      /* compress payload */
-      ASSH_RET_ON_ERR(k->cmp_algo->f_process(s->ctx, k->cmp_ctx, &p, s->tr_user_auth_done)
-		   | ASSH_ERRSV_DISCONNECT);
+      size_t mac_len = ma->mac_size + ca->auth_size;
+      uint_fast8_t align = assh_max_uint(ca->block_size, 8);
+      size_t os = s->queue_out_size - p->data_size;
 
-      if (p_ != p)
+      if (k->cmp_algo != &assh_compress_none)
 	{
-	  assh_queue_remove(&p_->entry);
-	  assh_packet_release(p_);
-	  assh_queue_push_front(q, &p->entry);
+	  /* compress payload */
+	  size_t tail_len = mac_len;
+
+	  switch (p->padding)
+	    {
+	    case ASSH_PADDING_MIN:
+	      tail_len += align + ASSH_PACKET_MIN_PADDING - 1;
+	      break;
+	    case ASSH_PADDING_MAX:
+	      tail_len += ASSH_PACKET_MAX_PADDING;
+	      break;
+	    default:
+	      ASSH_UNREACHABLE();
+	    }
+
+	  struct assh_packet_s *p_ = p;
+	  err = k->cmp_algo->f_process(s->ctx, k->cmp_ctx, &p,
+			       s->tr_user_auth_done, tail_len);
+
+	  switch (err)
+	    {
+	    case ASSH_OK:
+	      if (p_ != p)
+		{
+		  assh_queue_remove(&p_->entry);
+		  assh_packet_release(p_);
+		  assh_queue_push_front(q, &p->entry);
+		}
+	    case ASSH_NO_DATA:
+	      break;
+
+	    default:
+	      s->queue_out_size = os + p->data_size;
+	      ASSH_RETURN(err | ASSH_ERRSV_DISCONNECT);
+	    }
 	}
 
       /* compute various length and payload pointer values */
-      uint_fast8_t align = assh_max_uint(ca->block_size, 8);
-      size_t mac_len = ma->mac_size + ca->auth_size;
 
       size_t cipher_len = p->data_size;
       if (ma->etm || ca->auth_size)
@@ -513,14 +547,15 @@ assh_status_t assh_transport_write(struct assh_session_s *s,
       p->data_size += pad_len + mac_len;
       assert(p->data_size <= p->alloc_size);
 
+      s->queue_out_size = os + p->data_size;
+
       assh_store_u32(p->head.pck_len, p->data_size - 4 - mac_len);
       p->head.pad_len = pad_len;
       uint8_t *mac_ptr = p->data + p->data_size - mac_len;
       uint8_t *pad = mac_ptr - pad_len;
 
-      if (pad_len > 0)
-	ASSH_RET_ON_ERR(assh_prng_get(s->ctx, pad, pad_len, ASSH_PRNG_QUALITY_PADDING)
-		     | ASSH_ERRSV_DISCONNECT);
+      ASSH_RET_ON_ERR(assh_prng_get(s->ctx, pad, pad_len, ASSH_PRNG_QUALITY_PADDING)
+		      | ASSH_ERRSV_DISCONNECT);
 
       uint32_t seq = s->out_seq;
 
@@ -570,7 +605,7 @@ assh_status_t assh_transport_write(struct assh_session_s *s,
       s->stream_out_size = 0;
       *data = p->data;
       *size = p->data_size;
-      break;
+      return ASSH_OK;
     }
 
     /* the last packet buffer write was incomplete, yield to input */
@@ -579,7 +614,7 @@ assh_status_t assh_transport_write(struct assh_session_s *s,
 	  s->tr_st < ASSH_TR_DISCONNECT)
 	{
 	  ASSH_SET_STATE(s, stream_out_st, ASSH_TR_OUT_PACKETS_ENCIPHERED);
-	  return ASSH_OK;
+	  return ASSH_NO_DATA;
 	}
 
     /* the write stream buffer is an already enciphered output packet */
@@ -591,21 +626,12 @@ assh_status_t assh_transport_write(struct assh_session_s *s,
       *data = p->data + s->stream_out_size;
       *size = p->data_size - s->stream_out_size;
       ASSH_SET_STATE(s, stream_out_st, ASSH_TR_OUT_PACKETS_DONE);
-      break;
-    }
-
-    default:
-      ASSH_UNREACHABLE();
-
-    case ASSH_TR_OUT_CLOSED:
       return ASSH_OK;
     }
 
-  /* a buffer is available for output, return a write event */
-  e->id = ASSH_EVENT_WRITE;
-  e->f_done = &assh_event_write_done;
-  e->transport.write.transferred = 0;
-  return ASSH_OK;
+    default:
+      return ASSH_NO_DATA;
+    }
 }
 
 assh_bool_t
@@ -743,7 +769,7 @@ assh_status_t assh_transport_dispatch(struct assh_session_s *s,
   struct assh_packet_s *p = s->in_pck;
 
   /* test if a key re-exchange should have occured at this point */
-  ASSH_JMP_IF_TRUE(s->kex_bytes > ASSH_REKEX_THRESHOLD + CONFIG_ASSH_MAX_PAYLOAD * 16,
+  ASSH_JMP_IF_TRUE(s->kex_bytes > ASSH_REKEX_THRESHOLD + CONFIG_ASSH_MAX_PAYLOAD_LEN * 16,
 		   ASSH_ERR_PROTOCOL | ASSH_ERRSV_DISCONNECT, done);
 
   /* check protocol timeout */
@@ -967,3 +993,70 @@ assh_status_t assh_transport_dispatch(struct assh_session_s *s,
   ASSH_RETURN(err);
 }
 
+assh_status_t
+assh_transport_overhead(struct assh_session_s *s,
+			size_t *payload_size, size_t *packet_size)
+{
+  assh_status_t err;
+
+  struct assh_kex_keys_s *k = s->cur_keys_out;
+  const struct assh_algo_mac_s *ma = k->mac_algo;
+  const struct assh_algo_cipher_s *ca = k->cipher_algo;
+
+  uint_fast8_t align = assh_max_uint(ca->block_size, 8);
+  int_fast32_t mac_len = ma->mac_size + ca->auth_size;
+
+  if (*packet_size)
+    {
+      int_fast32_t head_len = ASSH_PACKET_HEADLEN;
+      int_fast32_t cipher_len = *packet_size - mac_len;
+
+      if (ma->etm || ca->auth_size)
+	{
+	  /* length field not enciphered with the payload */
+	  cipher_len -= 4;
+	  head_len -= 4;
+	}
+
+      ASSH_RET_IF_TRUE(cipher_len < (int_fast32_t)ca->block_size,
+		       ASSH_ERR_OUTPUT_OVERFLOW);
+
+      cipher_len -= cipher_len % align + /* minimal padding */ 4;
+
+      ASSH_RET_IF_TRUE(cipher_len <= head_len,
+		       ASSH_ERR_OUTPUT_OVERFLOW);
+
+      *payload_size = cipher_len - head_len;
+
+      return ASSH_OK;
+    }
+  else if (*payload_size)
+    {
+      uint_fast32_t head_len = 0;
+      uint_fast32_t cipher_len = ASSH_PACKET_HEADLEN + *payload_size;
+
+      if (ma->etm || ca->auth_size)
+	{
+	  /* length field not enciphered with the payload */
+	  cipher_len -= 4;
+	  head_len = 4;
+	}
+
+      size_t pad_len = align - cipher_len % align;
+      if (pad_len < 4)
+	pad_len += align;
+      cipher_len += pad_len;
+
+      *packet_size = head_len + cipher_len + mac_len;
+
+      return ASSH_OK;
+    }
+
+  ASSH_RETURN(ASSH_ERR_BAD_ARG);
+}
+
+size_t
+assh_transport_output_size(struct assh_session_s *s)
+{
+  return s->queue_out_size;
+}
