@@ -911,14 +911,13 @@ assh_save_pub_openssh(struct assh_context_s *c,
 		      const struct assh_key_s *head, FILE *file,
 		      const uint8_t *blob, size_t blob_len)
 {
-  assh_status_t err;
   struct asshh_base64_ctx_s b64;
-  size_t maxlen = asshh_base64_encoded_size(blob_len);
+  size_t maxlen = asshh_base64_max_encoded_size(blob_len);
   uint8_t tmp[maxlen];
 
   asshh_base64_init(&b64, tmp, maxlen);
-  ASSH_RET_ON_ERR(asshh_base64_encode_update(&b64, blob, blob_len));
-  ASSH_RET_ON_ERR(asshh_base64_encode_final(&b64));
+  ASSH_ASSERT(asshh_base64_encode_update(&b64, blob, blob_len));
+  ASSH_ASSERT(asshh_base64_encode_final(&b64));
 
   fputs(head->type, file);
   fputc(' ', file);
@@ -937,14 +936,13 @@ assh_save_rfc4716(struct assh_context_s *c,
 		  const uint8_t *blob,
 		  size_t blob_len)
 {
-  assh_status_t err;
   struct asshh_base64_ctx_s b64;
-  size_t maxlen = asshh_base64_encoded_size(blob_len);
+  size_t maxlen = asshh_base64_max_encoded_size(blob_len);
   uint8_t tmp[maxlen];
 
   asshh_base64_init(&b64, tmp, maxlen);
-  ASSH_RET_ON_ERR(asshh_base64_encode_update(&b64, blob, blob_len));
-  ASSH_RET_ON_ERR(asshh_base64_encode_final(&b64));
+  ASSH_ASSERT(asshh_base64_encode_update(&b64, blob, blob_len));
+  ASSH_ASSERT(asshh_base64_encode_final(&b64));
 
   size_t l = asshh_base64_outsize(&b64);
   char *s = (char*)tmp;
@@ -1027,26 +1025,28 @@ assh_save_rfc1421(struct assh_context_s *c,
         return err;
     }
 
-  /* base64 encode */
-  struct asshh_base64_ctx_s b64;
-  size_t maxlen = asshh_base64_encoded_size(blob_len);
-  uint8_t *tmp = alloca(maxlen);
-
-  asshh_base64_init(&b64, tmp, maxlen);
-  ASSH_RET_ON_ERR(asshh_base64_encode_update(&b64, blob, blob_len));
-  ASSH_RET_ON_ERR(asshh_base64_encode_final(&b64));
-
-  size_t l = asshh_base64_outsize(&b64);
-  char *s = (char*)tmp;
-
-  /* text output */
-  while (l)
     {
-      size_t r = l > 64 ? 64 : l;
-      fwrite(s, r, 1, file);
-      fputc('\n', file);
-      l -= r;
-      s += r;
+      /* base64 encode */
+      struct asshh_base64_ctx_s b64;
+      size_t maxlen = asshh_base64_max_encoded_size(blob_len);
+      uint8_t tmp[maxlen];
+
+      asshh_base64_init(&b64, tmp, maxlen);
+      ASSH_ASSERT(asshh_base64_encode_update(&b64, blob, blob_len));
+      ASSH_ASSERT(asshh_base64_encode_final(&b64));
+
+      size_t l = asshh_base64_outsize(&b64);
+      char *s = (char*)tmp;
+
+      /* text output */
+      while (l)
+        {
+          size_t r = l > 64 ? 64 : l;
+          fwrite(s, r, 1, file);
+          fputc('\n', file);
+          l -= r;
+          s += r;
+        }
     }
 
   fprintf(file, "-----%s %s KEY-----\n", "END", type);
@@ -1189,6 +1189,78 @@ asshh_key_load(struct assh_context_s *c,
   ASSH_RETURN(assh_key_load(c, key, algo, role, format, blob, blob_len));
 }
 
+assh_status_t
+asshh_key_load_base64(struct assh_context_s *c,
+		      struct assh_key_s **key,
+		      const char *key_algo,
+		      enum assh_algo_class_e role,
+		      enum assh_key_format_e format,
+		      const char *b64, size_t b64_len)
+{
+  assh_status_t err;
+  const struct assh_key_algo_s *algo = NULL;
+
+  ASSH_RET_IF_TRUE(key_algo && assh_key_algo_by_name(c, role,
+		     key_algo, strlen(key_algo), &algo),
+		   ASSH_ERR_MISSING_ALGO);
+
+  size_t sc_len = asshh_base64_max_decoded_size(b64_len);
+  ASSH_SCRATCH_ALLOC(c, uint8_t, sc, sc_len,
+		     ASSH_ERRSV_CONTINUE, err_);
+
+  struct asshh_base64_ctx_s bctx;
+  asshh_base64_init(&bctx, sc, sc_len);
+  ASSH_JMP_ON_ERR(asshh_base64_decode_update(&bctx, (uint8_t*)b64, b64_len), err_sc);
+  ASSH_JMP_ON_ERR(asshh_base64_decode_final(&bctx), err_sc);
+
+  const uint8_t *blob = sc;
+  ASSH_JMP_ON_ERR(assh_key_load(c, key, algo, role,
+		    format, &blob, asshh_base64_outsize(&bctx)), err_sc);
+
+  err = ASSH_OK;
+ err_sc:
+  ASSH_SCRATCH_FREE(c, sc);
+ err_:
+  return err;
+}
+
+ASSH_WARN_UNUSED_RESULT assh_status_t
+asshh_key_output_base64(struct assh_context_s *c,
+			const struct assh_key_s *key,
+			enum assh_key_format_e format,
+			char *b64, size_t *b64_len)
+{
+  assh_status_t err = ASSH_OK;
+  size_t blob_len;
+
+  if (!b64)
+    {
+      ASSH_RET_ON_ERR(assh_key_output(c, key, NULL, &blob_len, format));
+      *b64_len = asshh_base64_max_encoded_size(blob_len);
+      return ASSH_OK;
+    }
+
+  blob_len = asshh_base64_max_decoded_size(*b64_len);
+
+  ASSH_SCRATCH_ALLOC(c, uint8_t, sc, blob_len,
+		     ASSH_ERRSV_CONTINUE, err_);
+
+  ASSH_JMP_ON_ERR(assh_key_output(c, key, sc, &blob_len, format), err_sc);
+
+  struct asshh_base64_ctx_s bctx;
+  asshh_base64_init(&bctx, (uint8_t*)b64, *b64_len);
+  ASSH_ASSERT(asshh_base64_encode_update(&bctx, (uint8_t*)sc, blob_len));
+  ASSH_ASSERT(asshh_base64_encode_final(&bctx));
+
+  *b64_len = asshh_base64_outsize(&bctx);
+
+  err = ASSH_OK;
+ err_sc:
+  ASSH_SCRATCH_FREE(c, sc);
+ err_:
+  return err;
+}
+
 #ifdef CONFIG_ASSH_KEY_CREATE
 assh_status_t
 asshh_key_create(struct assh_context_s *c,
@@ -1304,8 +1376,8 @@ asshh_key_fingerprint(struct assh_context_s *c,
     case ASSH_FP_BASE64_SHA256: {
       struct asshh_base64_ctx_s bctx;
       asshh_base64_init(&bctx, (uint8_t*)buf, 44);
-      ASSH_JMP_ON_ERR(asshh_base64_encode_update(&bctx, hash, 32), err_sc);
-      ASSH_JMP_ON_ERR(asshh_base64_encode_final(&bctx), err_sc);
+      ASSH_ASSERT(asshh_base64_encode_update(&bctx, hash, 32));
+      ASSH_ASSERT(asshh_base64_encode_final(&bctx));
       buf[43] = 0;              /* drop '=' */
       break;
     }
