@@ -275,6 +275,8 @@ static const char *cipher_list_long[] =
   "aes256-ctr",
   "aes128-gcm@openssh.com",
   "aes256-gcm@openssh.com",
+  "aes128-ocb@libassh.org",
+  "aes256-ocb@libassh.org",
   "camellia128-cbc",
   "camellia192-cbc",
   "camellia256-cbc",
@@ -636,6 +638,9 @@ void test_loop(unsigned int seed,
 				   strcmp(kex->variant, ka->algo_wk.algo.variant)))
 		continue;
 
+	      if (!assh_algo_supported(*a))
+		continue;
+
 	      kex_done |= test_loop_2(seed, ka, kex, sign, cipher, mac, comp, cycles);
 	    }
 
